use reqwest::Error as ReqwestError;
use std::fmt::{self, Display};
use thiserror::Error;

pub mod discord;

#[derive(Debug, serde::Deserialize, PartialEq, Eq)]
pub struct DiscordErrorResponse {
    pub message: String,
    pub code: u16,
}

impl Display for DiscordErrorResponse {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "DiscordErrorResponse: {} ({})", self.message, self.code)
    }
}

#[derive(Debug, Error)]
pub enum ApiError {
    #[error("Reqwest error: {0}")]
    Reqwest(#[from] ReqwestError),
    #[error("API Error: {0}")]
    Api(DiscordErrorResponse),
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn display_discord_error_response() {
        let error = DiscordErrorResponse {
            message: "Message".into(),
            code: 42,
        };
        let error_str = error.to_string();
        let expected = "DiscordErrorResponse: Message (42)".to_string();
        assert_eq!(expected, error_str);
    }
}
